/************************************************************
** $Id: header.c,v 0.6 2003/12/15 15:03:33 bcx Exp bcx $
**
** Copyright Bryan Costales and Perforce Software, Inc. 2003
**
** This code is "open source" as defined by version 1.9 of
** the Open Source Definition from:
**
**	http://www.opensource.org/docs/definition.php.
**
** Entry points defined in this file:
**
** header_recipient():
** 	Add the passed recipients to a growing list of
** 	recipients (decomments each along the way).
** deRFC822():
** 	Strip the RFC822 comments from a recipient address.
** header_received():
** 	Check a received header to see if it was created by
**	one of our mx servers.
** header_messageid():
** 	Check a message-id header to see if it is legal.
*************************************************************/

# define EXTERN extern
# include "slow.h"

/*
** A simple minded count that avoids mallocs.
*/
int
header_recipient(char *value)
{
	char *vp;
	int count = 1;

	if (value == NULL)
		return 0;

	for (vp = value; *vp != '\0'; ++vp)
	{
		if (*vp == ',')
			++count;
	}
	return count;
}

/*
** A recipient can be buried among RFC822 comments.
** We snarf the address from the comments.
*/
char *
deRFC822(char *addr)
{
	int inquote;
	char *cp, *ep;

	if (addr == NULL)
		return NULL;

	/*
	 * First try to shortcut to an <addr> address.
	 */
	if ((cp = strchr(addr, '<')) != NULL)
	{
		++cp;
		ep = strchr(cp, '>');
		if (ep == NULL)
			return cp;
		*ep = '\0';
		while (*cp == '<')
			++cp;
		if (cp >= ep)
			return NULL;
		return cp;
	}

	/*
	 * Then remove any (comment) addr ...
	 * Include handling of nested comments (a(b))
	 */
	inquote = 0;
	for (cp = addr; *cp != '0'; ++cp)
	{
		if (*cp == '(')
		{
			++inquote;
			continue;
		}
		if (*cp == ')')
		{
			--inquote;
			continue;
		}
		if (inquote > 0)
			continue;
		if (isspace((int)*cp))
			continue;
		else
			break;
	}
	if (*cp == '\0')
		return NULL;
	for (ep = cp+1; *ep != '\0'; ++ep)
	{
		if (isspace((int)*ep))
			break;
	}
	*ep = '\0';
	return cp;
}

/*
** Process a received header to see if it was produced by one
** of our MX servers.
** If it was, return the IP number of the host that sent it to
** the MX server.
*/
PRIMARY_KEY
header_received(char *value)
{
	char **argv, **av;
	char *cp, *ep;
	PRIMARY_KEY ip;

	if (value == NULL)
		return (PRIMARY_KEY)0;


	argv = mkargv(value, ' ', TRUE);
	if (argv == NULL)
		return (PRIMARY_KEY)0;

	for (av = argv; *av != NULL; ++av)
	{
		if (strcasecmp(*av, "by") == 0)
			break;
	}
	if (*av == NULL || *(++av) == NULL)
	{
		argv = free_list(argv);
		return (PRIMARY_KEY)0;
	}

	if (is_in_list(*av, MxHosts) == FALSE)
	{
		argv = free_list(argv);
		return (PRIMARY_KEY)0;
	}

	/*
	 * This Received: header was created by
	 * one of our MX servers.
	 */
	cp = NULL;
	for (av = argv; *av != NULL; ++av)
	{
		if (strcasecmp(*av, "by") == 0)
		{
			argv = free_list(argv);
			return (PRIMARY_KEY)0;
		}
		if (strcasecmp(*av, "from") == 0)
			continue;
		if ((cp = strchr(*av, '[')) != NULL)
			break;
	}
	if (cp == NULL)
	{
		argv = free_list(argv);
		return (PRIMARY_KEY)0;
	}
	++cp;
	if ((ep = strchr(cp, ']')) == NULL)
	{
		argv = free_list(argv);
		return (PRIMARY_KEY)0;
	}
	*ep = '\0';

	if ((int)(ip = inet_addr(cp)) == -1)
	{
		argv = free_list(argv);
		return (PRIMARY_KEY)0;
	}

	argv = free_list(argv);
	ip = revbytes(ip);
	return ip;
}

/*
** The message-id: header must exist and be properly formed.
** If it exists, this routine will determine if it is properly
** formed. It must be inside angle braces and look like
** an address.
*/
bool
header_messageid(char *value)
{
	char *v, *e;

	if (value == NULL)
		return FALSE;

	v = cleanup(value);
	if (v == NULL)
		return FALSE;

	if (strchr(v, ' ') != NULL)
		return FALSE;

	if (strchr(v, '\t') != NULL)
		return FALSE;

	if (*v != '<')
		return FALSE;

	e = v + strlen(v) -1;
	if (*e != '>')
		return FALSE;

	if ((v = strchr(value, '@')) == NULL)
		return FALSE;
	if (strchr(++v, '.') == NULL)
		return FALSE;

	return TRUE;
}
